local core = LibStub("AceAddon-3.0"):GetAddon("SilverDragon")
local module = core:NewModule("Scan_Cache", "AceEvent-3.0", "AceConsole-3.0")

local globaldb
function module:OnInitialize()
	globaldb = core.db.global
end

function module:OnEnable()
	core.RegisterCallback(self, "Scan")
end

local already_cached = {}
local already_seen = {}
local first_cachescan = true
local cache_tooltip = CreateFrame("GameTooltip", "SDCacheTooltip")
cache_tooltip:AddFontStrings(
	cache_tooltip:CreateFontString("$parentTextLeft1", nil, "GameTooltipText"),
	cache_tooltip:CreateFontString("$parentTextRight1", nil, "GameTooltipText")
)

local function is_cached(id)
	-- this doesn't work with just clearlines and the setowner outside of this, and I'm not sure why
	cache_tooltip:SetOwner(WorldFrame, "ANCHOR_NONE")
	cache_tooltip:SetHyperlink(("unit:0xF53%05X00000000"):format(id))
	return cache_tooltip:IsShown()
end
module.already_cached = already_cached
module.already_seen = already_seen

function module:Scan(callback, zone)
	if not core.db.profile.cache then
		return
	end
	if first_cachescan then
		for mob, id in pairs(globaldb.mob_id) do
			if is_cached(id) then
				already_cached[id] = true
				already_seen[id] = true
			end
		end
		first_cachescan = false
		return
	end
	
	local zone_mobs = globaldb.mobs_byzoneid[zone]
	for id, lastseen in pairs(globaldb.mob_id) do
		if id and not already_cached[id] and is_cached(id) then--If not already cached in slver dragon.
			already_cached[id] = true--Add them to already cached immediately regardless of seen or other option right off. Need to sure all mobs get added to filter even if not in that zone, so if we cache them in diff zones they get filtered.
			if not zone_mobs then return end
			for id, lastseen in pairs(zone_mobs) do--Check whether they belong in this zone or not before firing seen event however.
				--Below used for savemob off cache, so we don't nil out current data with empty data when writing savemob off cache (unless they were already nil)
				local level = globaldb.mob_level[id] or nil
				local creature_type = globaldb.mob_type[id] or nil
				local elite = globaldb.mob_elite[id] or nil
				--End of Savemob info gathering.
				if (not globaldb.mob_tameable[id] or core.db.profile.cache_tameable) and is_cached(id) and not already_seen[id] then
					local current_zone, x, y = core:GetPlayerLocation()
					already_seen[id] = true
					if globaldb.mob_tameable[id] and id ~= 3868 then--If it's tamable (excluding blood seeker cause although he's tamable flagged, it's impossible to tame him. (he spawns about 100 yards off ground, and despawns after 3 seconds, no hunter is gonna pull that off)
						core:NotifyMob(id, globaldb.mob_name[id], current_zone, x, y, false, false, "cache", false)--Don't trust it's validity enough to save newloc
					else
						local newloc = core:SaveMob(current_zone, mob, x, y, level, elite, creature_type, id)--Save mob data, with new seen time location, while preserving the existing data we had as well.
						core:NotifyMob(id, globaldb.mob_name[id], current_zone, x, y, false, newloc, "cache", false)--If it's not tamable save the loc/seen information for it.
					end
				end
			end
		end
	end
end
core.RegisterCallback(module, "Import", function()
	if not first_cachescan then
		table.wipe(already_cached)
		first_cachescan = true
	end
end)

module:RegisterChatCommand("sdcached", function()
	local lookup = {}
	for mob, id in pairs(globaldb.mob_id) do
		lookup[id] = mob
	end
	local output
	module:Print("The following mobs are in the NPC cache, and so will not be detected by the cache scanner.")
	for id,_ in pairs(already_cached) do
		module:Print(" ", lookup[id])
		output = true
	end
	if not output then
		module:Print("Nothing")
	end
end)
